from steamship import File

from ..client.helpers import deploy_app, register_app_as_plugin, _steamship
from ..demo_apps.plugin_file_importer import TEST_DOC

__copyright__ = "Steamship"
__license__ = "MIT"


def test_e2e_importer():
    client = _steamship()
    with deploy_app("plugin_file_importer.py") as (app, version, instance):
        with register_app_as_plugin(client, "fileImporter", "do_import", app, instance) as plugin:
            file = File.create(
                client=client,
                name="Test.txt",
                content="This is a test.",
                plugin=plugin.handle
            ).data

            data = file.raw().data

            assert (data.decode('utf-8') == TEST_DOC)

            file.delete()



import contextlib
import io
import os
import random
import string
import zipfile

from steamship import App, AppVersion, AppInstance
from steamship import Steamship, EmbeddingIndex, File
from steamship.base import Client
from steamship.data.plugin import Plugin
from steamship.data.user import User
from steamship.data.corpus import Corpus

__copyright__ = "Steamship"
__license__ = "MIT"


def _env_or(env_var: str, or_val: str) -> str:
    if env_var in os.environ:
        return os.getenv(env_var)
    return or_val


def _random_name() -> str:
    letters = string.digits + string.ascii_letters
    id = ''.join(random.choice(letters) for i in range(10))
    return "test_{}".format(id)


_TEST_EMBEDDER = "test-embedder-v1"


@contextlib.contextmanager
def _random_index(steamship: Steamship, plugin: str = _TEST_EMBEDDER) -> EmbeddingIndex:
    index = steamship.create_index(
        plugin=plugin
    ).data
    yield index
    index.delete()  # or whatever you need to do at exit


@contextlib.contextmanager
def _random_file(steamship: Steamship, content: str = "") -> File:
    file = steamship.create_file(
        name=_random_name(),
        contents=content
    ).data
    yield file
    file.delete()  # or whatever you need to do at exit

@contextlib.contextmanager
def _random_corpus(client: Steamship) -> Corpus:
    corpus = Corpus.create(client, name=_random_name()).data
    yield corpus
    corpus.delete()

def _steamship() -> Steamship:
    # This should automatically pick up variables from the environment.
    return Steamship(profile="test")


def create_app_zip(filename) -> bytes:
    full_path = os.path.join(os.path.dirname(__file__), '..', 'demo_apps', filename)
    zip_buffer = io.BytesIO()

    files = []
    files.append(('api.py', io.BytesIO(open(full_path, 'rb').read())))

    # TODO: This is very dependent on the setup of the local machine.
    # Might be good to find a more machine-invariant solution here.
    # The goal is to copy in all the dependencies of the lambda package.
    # Which are: steamship (current repo), setuptools_scm, requests
    venv = os.path.join(os.path.dirname(__file__), '..', '..', '.venv', 'lib', 'python3.8', 'site-packages')

    pypi_zips = [
        os.path.join(os.path.dirname(__file__), '..', '..', 'src', 'steamship'),
        os.path.join(venv, 'setuptools_scm'),
        os.path.join(venv, 'requests'),
    ]

    with zipfile.ZipFile(zip_buffer, "a", zipfile.ZIP_DEFLATED, False) as zip_file:
        for file_name, data in files:
            zip_file.writestr(file_name, data.getvalue())

        for pypi_zip in pypi_zips:
            for root, dirs, files in os.walk(pypi_zip):
                for file in files:
                    zip_file.write(
                        os.path.join(root, file),
                        os.path.relpath(
                            os.path.join(root, file),
                            os.path.join(pypi_zip, '..')
                        )
                    )

    ret = zip_buffer.getvalue()

    with open("/tmp/{}.zip".format(filename), "wb") as f:  # use `wb` mode
        f.write(ret)

    return ret


@contextlib.contextmanager
def deploy_app(py_name: str):
    client = _steamship()
    name = _random_name()
    app = App.create(client, name=name)
    assert (app.error is None)
    assert (app.data is not None)
    app = app.data

    zip_bytes = create_app_zip(py_name)
    version = AppVersion.create(
        client,
        appId=app.id,
        filebytes=zip_bytes
    )
    version.wait()
    assert (version.error is None)
    assert (version.data is not None)
    version = version.data

    instance = AppInstance.create(
        client,
        appId=app.id,
        appVersionId=version.id,
    )
    instance.wait()
    assert (instance.error is None)
    assert (instance.data is not None)
    instance = instance.data

    assert (instance.appId == app.id)
    assert (instance.appVersionId == version.id)

    user = User.current(client).data

    assert (instance.userHandle == user.handle)
    assert (instance.userId == user.id)

    yield (app, version, instance)

    res = instance.delete()
    assert (res.error is None)

    res = version.delete()
    assert (res.error is None)

    res = app.delete()
    assert (res.error is None)


@contextlib.contextmanager
def register_app_as_plugin(client: Client, type: string, path: str, app: App, instance: AppInstance) -> Plugin:
    url = instance.full_url_for(
        path=path,
        appHandle=app.handle,
        useSubdomain=False  # In a test setting, the subdomain is hard to use
    )
    metadata = dict(
        http=dict(
            headers=dict(
                Authorization="Bearer {}".format(client.config.apiKey)
            )
        )
    )

    plugin = Plugin.create(
        client=client,
        name=instance.name,
        handle=instance.handle,
        description="Auto-generated",
        type=type,
        url=url,
        transport="jsonOverHttp",
        isPublic=True,
        metadata=metadata
    )
    assert (plugin.error is None)
    assert (plugin.data is not None)
    plugin = plugin.data

    yield plugin

    res = plugin.delete()
    assert (res.error is None)



from steamship.plugin.parser import ParseRequest, ParseResponse
from steamship.plugin.service import PluginRequest, PluginResponse

__copyright__ = "Steamship"
__license__ = "MIT"

from ..demo_apps.plugin_parser import TestParserPlugin

TEST_REQ = ParseRequest(
    docs=["Hi there."],
    blockIds=["ABC"]
)
TEST_PLUGIN_REQ = PluginRequest(data=TEST_REQ)
TEST_REQ_DICT = TEST_PLUGIN_REQ.to_dict()


def _test_resp(res):
    assert (type(res) == PluginResponse)
    assert (type(res.data) == ParseResponse)
    assert (len(res.data.blocks) == 1)
    assert (res.data.blocks[0].text == TEST_REQ.docs[0])


def test_parser():
    parser = TestParserPlugin()
    res = parser.run(TEST_PLUGIN_REQ)
    _test_resp(res)



from abc import ABC

from steamship.base import Client
from steamship.data.file import FileImportRequest, FileImportResponse
from steamship.plugin.service import PluginService, PluginRequest


class FileImporter(PluginService[FileImportRequest, FileImportResponse], ABC):
    @classmethod
    def subclass_request_from_dict(cls, d: any, client: Client = None) -> PluginRequest[FileImportRequest]:
        return FileImportRequest.from_dict(d, client=client)



from dataclasses import dataclass
from typing import List

from steamship.base import Client, Response, Request
from steamship.data.search import Hit


@dataclass
class ClassifierCreateRequest(Request):
    plugin: str
    name: str = None
    upsert: bool = True
    save: bool = True
    labels: List[str] = None


@dataclass
class ClassifierCreateResponse:
    classifierId: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "ClassifierCreateResponse":
        return ClassifierCreateResponse(
            classifierId=d.get('classifierId', None),
        )


@dataclass
class ClassifyRequest(Request):
    docs: List[str]
    classifierId: str = None
    plugin: str = None
    labels: List[str] = None
    k: int = None


@dataclass
class ClassifyResponse:
    classifierId: str = None
    plugin: str = None
    hits: List[List[Hit]] = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "ClassifyResponse":
        hits = [[Hit.from_dict(h) for h in innerList] for innerList in (d.get("hits", []) or [])]
        return ClassifyResponse(
            classifierId=d.get('classifierId', None),
            plugin=d.get('plugin', None),
            hits=hits
        )


@dataclass
class LabelInsertRequest(Request):
    value: str
    externalId: str = None
    externalType: str = None
    metadata: str = None


@dataclass
class LabelInsertResponse:
    labelId: str
    value: str
    externalId: str = None
    externalType: str = None
    metadata: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "LabelInsertResponse":
        return LabelInsertResponse(
            labelId=d.get('labelId', None),
            value=d.get('value', None),
            externalId=d.get('externalId', None),
            externalType=d.get('externalType', None),
            metadata=d.get('metadata', None),
        )


class Classifier:
    """A persistent, read-optimized index over embeddings.
    """

    def __init__(self, client: Client, id: str = None, name: str = None, plugin: str = None, labels: List[str] = None):
        if id is None and plugin is None:
            raise Exception("Either an ID or a plugin must be provided")

        self.client = client
        self.name = name
        self.id = id
        self.plugin = plugin
        self.labels = labels

    @staticmethod
    def create(
            client: Client,
            plugin: str,
            name: str = None,
            upsert: bool = True,
            save: bool = None,
            labels: List[str] = None,
            spaceId: str = None,
            spaceHandle: str = None
    ) -> "Response[Classifier]":
        if save == False:
            return Response(
                data=Classifier(client, id=None, plugin=plugin, name=name, labels=labels)
            )
        else:
            raise Exception("Persistent classifiers not yet supported.")
            req = ClassifierCreateRequest(
                plugin=plugin,
                name=name,
                upsert=upsert,
                save=save,
                labels=labels
            )
            res = client.post(
                'classifier/create',
                req,
                spaceId=spaceId,
                spaceHandle=spaceHandle
            )
            return ClassifierCreateResponse(
                client=client,
                name=req.name,
                id=res.data.get("classifierId", None)
            )

    def classify(
            self,
            docs: List[str],
            plugin: str = None,
            labels: List[str] = None,
            k: int = None,
            pd: bool = False,
            spaceId: str = None,
            spaceHandle: str = None
    ) -> Response[ClassifyResponse]:
        if self.id is None and self.plugin is None:
            raise Exception(
                "Neither an ID nor a plugin was found on the classifier object. Please reinitialize with one or the other.")

        if self.id is None and (labels is None or len(labels) == 0) and (self.labels is None or len(self.labels) == 0):
            raise Exception(
                'Since you are calling a stateless classifier, please include output labels in your classify request.')

        if self.id is not None and labels is not None and len(labels) > 0:
            raise Exception(
                "Since you are calling a stateful classifier, you can not include in-line labels in your classify request. Please add them first.")

        req = ClassifyRequest(
            docs=docs,
            classifierId=self.id,
            plugin=plugin if plugin is not None else self.plugin,
            labels=labels if (labels is not None and len(labels) > 0) else self.labels,
            k=k
        )
        ret = self.client.post(
            'classifier/classify',
            req,
            expect=ClassifyResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle
        )
        if pd is False:
            return ret

        import pandas as pd  # type: ignore
        return pd.DataFrame([(hit.score, hit.value) for hit in ret.data.hits[0]], columns=['Score', 'Value'])



from abc import ABC

from steamship.base import Client
from steamship.data.converter import ConvertRequest, ConvertResponse
from steamship.plugin.service import PluginService, PluginRequest


class Converter(PluginService[ConvertRequest, ConvertResponse], ABC):
    @classmethod
    def subclass_request_from_dict(cls, d: any, client: Client = None) -> PluginRequest[ConvertRequest]:
        return ConvertRequest.from_dict(d, client=client)



import json
from dataclasses import dataclass
from typing import List

from steamship.base import Client, Request
from steamship.data.block import Block


@dataclass
class TagResponse:
    blocks: List[Block] = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "TagResponse":
        blocks = [Block.from_dict(h) for h in (d.get("blocks", []) or [])]
        return TagResponse(
            blocks=blocks
        )


@dataclass
class TagRequest(Request):
    blocks: List[str] = None
    fileId: str = None
    parsedBlocks: List[Block] = None
    plugin: str = None
    metadata: any = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "TagRequest":
        metadata = d.get("metadata", None)
        if metadata is not None:
            try:
                metadata = json.loads(metadata)
            except:
                pass

        parsedBlocks = [Block.from_dict(dd) for dd in (d.get("parsedBlocks", []) or [])]

        return TagRequest(
            blocks=(d.get("blocks", []) or []),
            fileId=d.get('fileId', None),
            parsedBlocks=parsedBlocks,
            plugin=d.get("plugin", None),
            metadata=metadata
        )



from dataclasses import dataclass

from steamship.base import Client


@dataclass
class Span:
    client: Client = None
    id: str = None
    blockId: str = None
    type: str = None
    label: str = None
    text: str = None
    startCharIdx: int = None
    endCharIdx: int = None
    startTokenIdx: int = None
    endTokenIdx: int = None

    @staticmethod
    def from_dict(d: any, client: Client) -> "Span":
        if d is None:
            return None
        if 'span' in d:
            d = d['span']
        return Span(
            client=client,
            id=d.get('id', None),
            blockId=d.get('blockId', None),
            type=d.get('type', None),
            label=d.get('label', None),
            text=d.get('text', None),
            startCharIdx=d.get('startCharIdx', None),
            endCharIdx=d.get('endCharIdx', None),
            startTokenIdx=d.get('startTokenIdx', None),
            endTokenIdx=d.get('endTokenIdx', None)
        )

    def to_dict(self) -> dict:
        return dict(
            id=self.id,
            blockId=self.blockId,
            type=self.type,
            label=self.label,
            text=self.text,
            startCharIdx=self.startCharIdx,
            endCharIdx=self.endCharIdx,
            startTokenIdx=self.startTokenIdx,
            endTokenIdx=self.endTokenIdx,
        )



from dataclasses import dataclass
from typing import List, Dict

from steamship.base import Client, Request
from steamship.data.search import Hit


@dataclass
class EmbedRequest:
    docs: List[str]
    plugin: str
    metadata: Dict = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "EmbedRequest":
        return EmbedRequest(
            docs=d.get('docs', None),
            plugin=d.get('plugin', None),
            metadata=d.get('metadata', {})
        )


@dataclass
class EmbedResponse:
    embeddings: List[List[float]]

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "EmbedResponse":
        return EmbedResponse(
            embeddings=d.get('embeddings', None),
        )

    def to_dict(self) -> dict:
        return dict(
            embeddings=self.embeddings
        )


@dataclass
class EmbedAndSearchRequest(Request):
    query: str
    docs: List[str]
    plugin: str
    k: int = 1


@dataclass
class EmbedAndSearchResponse(Request):
    hits: List[Hit] = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "EmbedAndSearchResponse":
        hits = [Hit.from_dict(h) for h in (d.get("hits", []) or [])]
        return EmbedAndSearchResponse(
            hits=hits
        )



from steamship import BlockTypes, MimeTypes
from steamship.base.response import TaskStatus

from steamship.data.file import FileImportResponse

from .helpers import _random_name, _steamship

import json

__copyright__ = "Steamship"
__license__ = "MIT"


def test_file_upload():
    steamship = _steamship()
    name_a = "{}.mkd".format(_random_name())
    a = steamship.upload(
        name=name_a,
        content="A",
        mimeType=MimeTypes.MKD
    ).data
    assert (a.id is not None)
    assert (a.name == name_a)
    assert (a.mimeType == MimeTypes.MKD)

    name_b = "{}.txt".format(_random_name())
    b = steamship.upload(
        name=name_b,
        content="B",
        mimeType=MimeTypes.TXT
    ).data
    assert (b.id is not None)
    assert (b.name == name_b)
    assert (b.mimeType == MimeTypes.TXT)
    assert (a.id != b.id)

    name_c = "{}.txt".format(_random_name())
    c = steamship.upload(
        name=name_c,
        content="B",
        mimeType=MimeTypes.MKD
    ).data
    assert (c.mimeType == MimeTypes.MKD)  # The specified format gets precedence over filename

    d = steamship.upload(
        name=name_c,
        content="B",
    ).data
    assert (d.mimeType == MimeTypes.TXT)  # The filename is used in a pinch.

    a.delete()
    b.delete()
    c.delete()
    d.delete()


def test_file_scrape():
    steamship = _steamship()

    name_a = "{}.html".format(_random_name())
    a = steamship.scrape(
        name=name_a,
        url="https://edwardbenson.com/2020/10/gpt3-travel-agent"
    ).data
    assert (a.id is not None)
    assert (a.name == name_a)
    assert (a.mimeType == MimeTypes.HTML)

    name_b = "{}.html".format(_random_name())
    b = steamship.scrape(
        name=name_b,
        url="https://edwardbenson.com/2018/09/case-of-the-murderous-ai"
    ).data
    assert (b.id is not None)
    assert (a.id != b.id)
    assert (b.name == name_b)
    assert (b.mimeType == MimeTypes.HTML)

    a.delete()
    b.delete()


# def test_file_add_bloc():
#     steamship = _steamship()

#     name_a = "{}.txt".format(_random_name())
#     a = steamship.upload(
#         name=name_a,
#         content="This is a test."
#     )
#     assert(a.id is not None)
#     task  = a.convert()
#     task._run_development_mode()
#     task.wait()
#     q1 = a.query()
#     assert(len(q1.blocks) == 2)

#     # TODO: Append Content
#     # TODO: Append Blocks

def test_file_upload_then_parse():
    steamship = _steamship()

    name_a = "{}.txt".format(_random_name())
    a = steamship.upload(
        name=name_a,
        content="This is a test."
    ).data
    assert (a.id is not None)

    q1 = a.query().data
    assert (len(q1.blocks) == 0)

    task = a.convert()
    assert (task.error is None)
    assert (task.task is not None)
    assert (task.task.taskStatus == TaskStatus.waiting)

    task.wait()
    assert (task.error is None)
    assert (task.task is not None)
    assert (task.task.taskStatus == TaskStatus.succeeded)

    q1 = a.query().data
    assert (len(q1.blocks) == 2)
    assert (q1.blocks[0].type == BlockTypes.Document)
    assert (q1.blocks[1].type == BlockTypes.Paragraph)
    assert (q1.blocks[1].text == 'This is a test.')

    name_b = "{}.mkd".format(_random_name())
    b = steamship.upload(
        name=name_b,
        content="""# Header

This is a test."""
    ).data
    assert (b.id is not None)

    q1 = b.query().data
    assert (len(q1.blocks) == 0)

    task = b.convert()
    task.wait()

    q1 = b.query().data
    assert (len(q1.blocks) == 3)
    assert (q1.blocks[0].type == BlockTypes.Document)
    assert (q1.blocks[2].type == BlockTypes.Paragraph)
    assert (q1.blocks[2].text == 'This is a test.')
    assert (q1.blocks[1].type == BlockTypes.H1)
    assert (q1.blocks[1].text == 'Header')

    q2 = b.query(blockType=BlockTypes.H1).data
    assert (len(q2.blocks) == 1)
    assert (q2.blocks[0].type == BlockTypes.H1)
    assert (q2.blocks[0].text == 'Header')

    a.delete()
    b.delete()


def test_file_import_response_dict():
    resp = FileImportResponse(bytes=b'some bytes', mimeType=MimeTypes.BINARY)
    to_dict = resp.to_dict()
    from_dict = FileImportResponse.from_dict(to_dict)
    assert (resp.data == from_dict.data)
    assert (resp.mimeType == from_dict.mimeType)


def test_file_import_response_bytes_serialization():
    file_resp = FileImportResponse(bytes=b'some bytes', mimeType=MimeTypes.BINARY)
    to_dict = file_resp.to_dict()
    as_json_string = json.dumps(to_dict)
    as_dict_again = json.loads(as_json_string)
    assert (as_dict_again == to_dict)



from steamship import Steamship
from steamship.app import get, App, Response, Error, post, create_handler

class TestApp(App):
    def __init__(self, client: Steamship):
        # In production, the lambda handler will provide an Steamship client:
        # - Authenticated to the appropriate user
        # - Bound to the appropriate space
        self.client = client

        # Create an embedding index using (for now!) our
        # mock embedder.
        #
        # Note that the *scope* of this index is limited to the space
        # this app is executing within. Each new instance of the app
        # will resultingly have a fresh index.
        self.index = self.client.create_index(
            handle="qa-index",
            plugin="test-embedder-v1"
        ).data

    @get('greet')
    def greet1(self, name: str = "Person") -> Response:
        return Response(string='Hello, {}!'.format(name))

    @post('greet')
    def greet2(self, name: str = "Person") -> Response:
        return Response(string='Hello, {}!'.format(name))

    @get('space')
    def space(self) -> Response:
        return Response(string=self.client.config.spaceId)

    @get('config')
    def config(self) -> Response:
        return Response(json=dict(
            spaceId=self.client.config.spaceId,
            appBase=self.client.config.appBase,
            Client=self.client.config.apiBase,
            apiKey=self.client.config.apiKey
        ))

    @post('learn')
    def learn(self, fact: str = None) -> Response:
        """Learns a new fact."""
        if fact is None:
            return Error(message="Empty fact provided to learn.")

        if self.index is None:
            return Error(message="Unable to initialize QA index.")

        res = self.index.embed(fact)

        if res.error:
            # Steamship error messages can be passed straight
            # back to the user
            return Error(
                message=res.error.message,
                suggestion=res.error.suggestion,
                code=res.error.code
            )

        return Response(json=res.data)

    @post('query')
    def query(self, query: str = None, k: int = 1) -> Response:
        """Learns a new fact."""
        if query is None:
            return Error(message="Empty query provided.")

        if self.index is None:
            return Error(message="Unable to initialize QA index.")

        res = self.index.query(query=query, k=k)

        if res.error:
            # Steamship error messages can be passed straight
            # back to the user
            return Error(
                message=res.error.message,
                suggestion=res.error.suggestion,
                code=res.error.code
            )

        return Response(json=res.data)


handler = create_handler(TestApp)



from dataclasses import dataclass

from steamship.base import Client


@dataclass
class Token:
    client: Client = None
    id: str = None
    blockId: bool = None
    text: str = None
    textWithWs: str = None
    whitespace: str = None
    head: str = None
    leftEdge: str = None
    rightEdge: str = None
    entType: str = None
    entIob: str = None
    lemma: str = None
    normalized: str = None
    shape: str = None
    prefix: str = None
    suffix: str = None
    isAlpha: bool = None
    isAscii: bool = None
    isDigit: bool = None
    isTitle: bool = None
    isPunct: bool = None
    isLeftPunct: bool = None
    isRightPunct: bool = None
    isSpace: bool = None
    isBracket: bool = None
    isQuote: bool = None
    isCurrency: bool = None
    likeUrl: bool = None
    likeNum: bool = None
    likeEmail: bool = None
    isOov: bool = None
    isStop: bool = None
    pos: str = None
    tag: str = None
    dep: str = None
    lang: str = None
    prob: float = None
    charIndex: int = None
    tokenIndex: int = None
    parentIndex: int = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "Token":
        return Token(
            client=client,
            id=d.get("id", None),
            blockId=d.get("blockId", None),
            text=d.get("text", None),
            textWithWs=d.get("textWithWs", None),
            whitespace=d.get("whitespace", None),
            head=d.get("head", None),
            leftEdge=d.get("leftEdge", None),
            rightEdge=d.get("rightEdge", None),
            entType=d.get("entType", None),
            entIob=d.get("entIob", None),
            lemma=d.get("lemma", None),
            normalized=d.get("normalized", None),
            shape=d.get("shape", None),
            prefix=d.get("prefix", None),
            suffix=d.get("suffix", None),
            isAlpha=d.get("isAlpha", None),
            isAscii=d.get("isAscii", None),
            isDigit=d.get("isDigit", None),
            isTitle=d.get("isTitle", None),
            isPunct=d.get("isPunct", None),
            isLeftPunct=d.get("isLeftPunct", None),
            isRightPunct=d.get("isRightPunct", None),
            isSpace=d.get("isSpace", None),
            isBracket=d.get("isBracket", None),
            isQuote=d.get("isQuote", None),
            isCurrency=d.get("isCurrency", None),
            likeUrl=d.get("likeUrl", None),
            likeNum=d.get("likeNum", None),
            likeEmail=d.get("likeEmail", None),
            isOov=d.get("isOov", None),
            isStop=d.get("isStop", None),
            pos=d.get("pos", None),
            tag=d.get("tag", None),
            dep=d.get("dep", None),
            lang=d.get("lang", None),
            prob=d.get("prob", None),
            charIndex=d.get("charIndex", None),
            tokenIndex=d.get("tokenIndex", None),
            parentIndex=d.get("parentIndex", None)
        )

    def __len__(self):
        if self.text is None:
            return None
        return self.text.__len__

    @staticmethod
    def from_spacy(t: any, includeParseData: bool = True) -> "Token":
        if includeParseData is False:
            return Token(
                text=t.text,
                textWithWs=t.text_with_ws,
                whitespace=t.whitespace_,
                leftEdge=t.left_edge.text,
                rightEdge=t.right_edge.text,
                lemma=t.lemma_,
                isOov=t.is_oov,
                isStop=t.is_stop,
                charIndex=t.idx,
                tokenIndex=t.i,
            )

        return Token(
            text=t.text,
            textWithWs=t.text_with_ws,
            whitespace=t.whitespace_,
            head=t.head.text,
            leftEdge=t.left_edge.text,
            rightEdge=t.right_edge.text,
            entType=t.ent_type_,
            entIob=t.ent_iob_,
            lemma=t.lemma_,
            normalized=t.norm_,
            shape=t.shape_,
            prefix=t.prefix_,
            suffix=t.suffix_,
            isAlpha=t.is_alpha,
            isAscii=t.is_ascii,
            isDigit=t.is_digit,
            isTitle=t.is_title,
            isPunct=t.is_punct,
            isLeftPunct=t.is_left_punct,
            isRightPunct=t.is_right_punct,
            isSpace=t.is_space,
            isBracket=t.is_bracket,
            isQuote=t.is_quote,
            isCurrency=t.is_currency,
            likeUrl=t.like_url,
            likeNum=t.like_num,
            likeEmail=t.like_email,
            isOov=t.is_oov,
            isStop=t.is_stop,
            pos=t.pos_,
            tag=t.tag_,
            dep=t.dep_.upper(),
            lang=t.lang_,
            prob=t.prob,
            charIndex=t.idx,
            tokenIndex=t.i,
            parentIndex=t.head.i
        )

    def to_dict(self) -> dict:
        return dict(
            id=self.id,
            blockId=self.blockId,
            text=self.text,
            textWithWs=self.textWithWs,
            whitespace=self.whitespace,
            head=self.head,
            parentIndex=self.parentIndex,
            leftEdge=self.leftEdge,
            rightEdge=self.rightEdge,
            entType=self.entType,
            entIob=self.entIob,
            lemma=self.lemma,
            normalized=self.normalized,
            shape=self.shape,
            prefix=self.prefix,
            suffix=self.suffix,
            isAlpha=self.isAlpha,
            isAscii=self.isAscii,
            isDigit=self.isDigit,
            isTitle=self.isTitle,
            isPunct=self.isPunct,
            isLeftPunct=self.isLeftPunct,
            isRightPunct=self.isRightPunct,
            isSpace=self.isSpace,
            isBracket=self.isBracket,
            isQuote=self.isQuote,
            isCurrency=self.isCurrency,
            likeUrl=self.likeUrl,
            likeNum=self.likeNum,
            likeEmail=self.likeEmail,
            isOov=self.isOov,
            isStop=self.isStop,
            pos=self.pos,
            tag=self.tag,
            dep=self.dep,
            lang=self.lang,
            prob=self.prob,
            charIndex=self.charIndex,
            tokenIndex=self.tokenIndex
        )



from abc import ABC

from steamship.base import Client
from steamship.data.parser import ParseRequest, ParseResponse
from steamship.plugin.service import PluginService, PluginRequest


class Parser(PluginService[ParseRequest, ParseResponse], ABC):
    @classmethod
    def subclass_request_from_dict(cls, d: any, client: Client = None) -> PluginRequest[ParseRequest]:
        return ParseRequest.from_dict(d, client=client)



from steamship import File, Corpus

from ..client.helpers import deploy_app, register_app_as_plugin, _steamship, _random_corpus
from ..demo_apps.plugin_file_importer import TEST_DOC

__copyright__ = "Steamship"
__license__ = "MIT"


def test_e2e_corpus_importer():
    client = _steamship()
    with deploy_app("plugin_corpus_importer.py") as (app, version, instance):
        with register_app_as_plugin(client, "corpusImporter", "do_import", app, instance) as plugin:
            with _random_corpus(client) as corpus:
                resp = corpus.doImport(
                    plugin=plugin.handle,
                    value="dummy-value"
                )
                resp.wait()

                # We should now have two files!
                files = File.list(client, corpusId=corpus.id).data
                assert (files.files is not None)
                assert (len(files.files) == 2)

                for file in files.files:
                    data = file.raw().data
                    assert (data.decode('utf-8') == TEST_DOC)
                    file.delete()



import json
from dataclasses import dataclass
from typing import Any, List

from steamship.base import Client, Request, Response, str_to_metadata
from steamship.base.request import IdentifierRequest
from steamship.data.file import File, FileImportRequest, ListFilesResponse


@dataclass
class CreateCorpusRequest(Request):
    corpusId: str = None
    name: str = None
    handle: str = None
    description: str = None
    externalId: str = None
    externalType: str = None
    isPublic: bool = None
    metadata: str = None
    upsert: bool = None


@dataclass
class DeleteCorpusRequest(Request):
    corpusId: str


@dataclass
class ListPublicCorporaRequest(Request):
    pass


@dataclass
class ListPrivateCorporaRequest(Request):
    pass


@dataclass
class CorpusImportRequest:
    # The Corpus Identifiers
    client: Client = None
    id: str = None
    handle: str = None
    type: str = 'corpus'

    # Data for the plugin
    value: str = None
    data: str = None
    url: str = None
    plugin: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "CorpusImportRequest":
        return CorpusImportRequest(
            client=client,
            id=d.get('id', None),
            handle=d.get('handle', None),
            type='corpus',
            value=d.get('value', None),
            data=d.get('data', None),
            url=d.get('url', None),
            plugin=d.get('plugin', None)
        )

    def to_dict(self) -> dict:
        return dict(
            id=self.id,
            handle=self.handle,
            type=self.type,
            value=self.value,
            data=self.data,
            url=self.url,
            plugin=self.plugin
        )


@dataclass
class CorpusImportResponse:
    client: Client = None
    fileImportRequests: List[FileImportRequest] = None

    def __init__(
            self,
            fileImportRequests: List[FileImportRequest] = None
    ):
        self.fileImportRequests = fileImportRequests

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "CorpusImportResponse":
        return CorpusImportRequest(
            client=client,
            fileImportRequests=[FileImportRequest.from_dict(req) for req in d.get('fileImportRequests', [])]
        )

    def to_dict(self) -> "CorpusImportResponse":
        return dict(
            fileImportRequests=self.fileImportRequests
        )


@dataclass
class Corpus:
    """A corpus of files.
    """
    client: Client
    id: str = None
    name: str = None
    handle: str = None
    description: str = None
    externalId: str = None
    externalType: str = None
    metadata: Any = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "Corpus":
        if 'corpus' in d:
            d = d['corpus']

        return Corpus(
            client=client,
            id=d.get('id', None),
            name=d.get('name', None),
            handle=d.get('handle', None),
            description=d.get('description', None),
            externalId=d.get('externalId', None),
            externalType=d.get('externalType', None),
            metadata=str_to_metadata(d.get("metadata", None)),
        )

    @staticmethod
    def default(
            client: Client,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ) -> "Response[Corpus]":
        req = IdentifierRequest()
        return client.post(
            'corpus/get',
            req,
            expect=Corpus,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def doImport(
            self,
            value: str = None,
            url: str = None,
            plugin: str = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ) -> "Response[CorpusImportResponse]":
        req = CorpusImportRequest(
            type="corpus",
            id=self.id,
            value=value,
            url=url,
            plugin=plugin
        )
        return self.client.post(
            'plugin/importCorpus',
            req,
            expect=CorpusImportResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    @staticmethod
    def create(
            client: Client,
            name: str,
            handle: str = None,
            description: str = None,
            externalId: str = None,
            externalType: str = None,
            metadata: any = None,
            isPublic: bool = False,
            upsert: bool = True,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ) -> "Response[Corpus]":
        if isinstance(metadata, dict) or isinstance(metadata, list):
            metadata = json.dumps(metadata)

        req = CreateCorpusRequest(
            name=name,
            handle=handle,
            description=description,
            isPublic=isPublic,
            upsert=upsert,
            externalId=externalId,
            externalType=externalType,
            metadata=metadata
        )
        return client.post(
            'corpus/create',
            req,
            expect=Corpus,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def delete(
            self,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> Response["Corpus"]:
        return self.client.post(
            'corpus/delete',
            IdentifierRequest(id=self.id),
            expect=Corpus,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def upload(
            self,
            filename: str = None,
            name: str = None,
            content: str = None,
            mimeType: str = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ) -> Response[File]:

        return File.upload(
            client=self.client,
            corpusId=self.id,
            filename=filename,
            name=name,
            content=content,
            mimeType=mimeType,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def scrape(
            self,
            url: str,
            name: str = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> File:

        return File.scrape(
            client=self.client,
            corpusId=self.id,
            url=url,
            name=name,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def list_files(
            self,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> ListFilesResponse:
        return File.list(
            client=self.client,
            corpusId=self.id,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )


@dataclass
class ListCorporaResponse(Request):
    corpora: List[Corpus]

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "ListCorporaResponse":
        return ListCorporaResponse(
            plugins=[Corpus.from_dict(x) for x in (d.get("corpus", []) or [])]
        )



from steamship import BlockTypes, MimeTypes
from steamship.base import Client
from steamship.base.response import TaskStatus

from .helpers import _random_name, _steamship

__copyright__ = "Steamship"
__license__ = "MIT"


# TODO: It should fail if the docs field is empty.
# TODO: It should fail if the file hasn't been converted.

def parse_file(client: Client, parserModel: str):
    name_a = "{}.mkd".format(_random_name())
    T = "A Poem"
    P1_1 = "Roses are red."
    P1_2 = "Violets are blue."
    P2_1 = "Sugar is sweet, and I love you."

    CONTENT = "# {}\n\n{} {}\n\n{}".format(T, P1_1, P1_2, P2_1)

    a = client.upload(
        name=name_a,
        content=CONTENT,
        mimeType=MimeTypes.MKD
    ).data
    assert (a.id is not None)
    assert (a.name == name_a)
    assert (a.mimeType == MimeTypes.MKD)

    a.convert().wait()

    raw = a.raw()
    assert (raw.data.decode('utf-8') == CONTENT)

    q1 = a.query(blockType=BlockTypes.H1).data
    assert (len(q1.blocks) == 1)
    assert (q1.blocks[0].type == BlockTypes.H1)
    assert (q1.blocks[0].text == T)

    q2 = a.query(blockType=BlockTypes.Paragraph).data
    assert (len(q2.blocks) == 2)
    assert (q2.blocks[0].type == BlockTypes.Paragraph)
    assert (q2.blocks[0].text == "{} {}".format(P1_1, P1_2))

    # The sentences aren't yet parsed out!
    q2 = a.query(blockType=BlockTypes.Sentence).data
    assert (len(q2.blocks) == 0)

    # Now we parse
    task = a.parse(plugin="test-parser-v1")
    assert (task.error is None)
    assert (task.task is not None)
    assert (task.task.taskStatus == TaskStatus.waiting)

    task.wait()
    assert (task.error is None)
    assert (task.task is not None)
    assert (task.task.taskStatus == TaskStatus.succeeded)

    # Now the sentences should be parsed!
    q2 = a.query(blockType=BlockTypes.Sentence).data
    assert (len(q2.blocks) == 2)  # The 5th is inside the header!

    a.clear()

    q2 = a.query(blockType=BlockTypes.Sentence).data
    assert (len(q2.blocks) == 0)  # The 5th is inside the header!

    a.delete()


def test_parse_file():
    steamship = _steamship()
    parse_file(steamship, "test-parser-v1")



import base64
import logging
from dataclasses import dataclass
from typing import Dict, Any

from steamship import MimeTypes
from steamship.base import Client
from steamship.data.block import Block


@dataclass
class ClientsideConvertRequest:
    type: str = None
    plugin: str = None
    id: str = None
    handle: str = None
    name: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "ClientsideConvertRequest":
        return ClientsideConvertRequest(
            type=d.get('type', None),
            plugin=d.get('plugin', None),
            id=d.get('id', None),
            handle=d.get('handle', None),
            name=d.get('name', None)
        )


@dataclass
class ConvertResponse():
    root: Block = None

    @staticmethod
    def from_dict(d: any = None, client: Client = None) -> "ConvertResponse":
        if d is None:
            return None

        return ConvertResponse(
            root=Block.from_dict(d.get('root', None), client=client)
        )

    def to_dict(self) -> Dict:
        if self.root is None:
            return dict()
        return dict(root=self.root.to_dict())


TEXT_MIME_TYPES = [
    MimeTypes.TXT,
    MimeTypes.MKD,
    MimeTypes.HTML,
    MimeTypes.DOCX,
    MimeTypes.PPTX
]


@dataclass
class ConvertRequest:
    plugin: str = None
    data: Any = None
    defaultMimeType: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "ConvertRequest":
        logging.info("ConvertRequest.fromDict {} {}".format(type(d), d))
        data = d.get('data', None)
        if data is not None and d.get('isBase64', False):
            data_bytes = base64.b64decode(data)
            if d.get('defaultMimeType', None) in TEXT_MIME_TYPES:
                data = data_bytes.decode('utf-8')
            else:
                data = data_bytes

        return ConvertRequest(
            plugin=d.get('plugin', None),
            data=data,
            defaultMimeType=d.get('defaultMimeType', None)
        )



from dataclasses import dataclass, asdict
from typing import List, Dict

from steamship.base import Client
from steamship.data.block import Block

MatcherClause = Dict[str, any]
Matcher = List[MatcherClause]


@dataclass
class TokenMatcher():
    label: str = None
    patterns: List[Matcher] = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "TokenMatcher":
        return TokenMatcher(
            label=d.get("label", None),
            patterns=(d.get("patterns", []) or [])
        )


@dataclass
class PhraseMatcher():
    label: str = None
    phrases: List[str] = None
    attr: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "PhraseMatcher":
        return PhraseMatcher(
            label=d.get("label", None),
            phrases=(d.get("phrases", []) or []),
            attr=d.get("attr", None)
        )


@dataclass
class DependencyMatcher():
    label: str = None
    # Note: Add a LABEL field to the matcher clause to have a token
    # get labeled in the response.
    patterns: List[Matcher] = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "DependencyMatcher":
        return DependencyMatcher(
            label=d.get("label", None),
            patterns=(d.get("patterns", []) or [])
        )


@dataclass
class ParseRequest:
    type: str = None
    id: str = None
    name: str = None
    handle: str = None
    docs: List[str] = None
    blockIds: List[str] = None
    plugin: str = None
    tokenMatchers: List[TokenMatcher] = None
    phraseMatchers: List[PhraseMatcher] = None
    dependencyMatchers: List[DependencyMatcher] = None
    includeTokens: bool = True
    includeParseData: bool = True
    includeEntities: bool = False
    metadata: any = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "ParseRequest":
        token_matchers = [TokenMatcher.from_dict(h) for h in (d.get("tokenMatchers", []) or [])]
        phrase_matchers = [PhraseMatcher.from_dict(h) for h in (d.get("phraseMatchers", []) or [])]
        dependency_matchers = [DependencyMatcher.from_dict(h) for h in (d.get("dependencyMatchers", []) or [])]

        includeTokens = d.get("includeTokens", True)
        if includeTokens is None:
            includeTokens = True

        includeParseData = d.get("includeParseData", True)
        if includeParseData is None:
            includeParseData = True

        includeEntities = d.get("includeEntities", True)
        if includeEntities is None:
            includeEntities = True

        metadata = d.get("metadata", None)
        if metadata is not None:
            try:
                metadata = json.loads(metadata)
            except:
                pass

        return ParseRequest(
            docs=(d.get("docs", []) or []),
            blockIds=(d.get("blockIds", []) or []),
            plugin=d.get("plugin", None),
            tokenMatchers=token_matchers,
            phraseMatchers=phrase_matchers,
            dependencyMatchers=dependency_matchers,
            includeTokens=includeTokens,
            includeParseData=includeParseData,
            includeEntities=includeEntities,
            metadata=metadata
        )

    def to_dict(self) -> dict:
        return asdict(self)


@dataclass
class ParseResponse:
    blocks: List[Block] = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "ParseResponse":
        blocks = [Block.from_dict(h, client=client) for h in (d.get("blocks", []) or [])]
        return ParseResponse(
            blocks=blocks
        )

    def to_dict(self) -> dict:
        blocks = None
        if self.blocks is not None:
            blocks = [block.to_dict() for block in self.blocks]

        return dict(
            blocks=blocks
        )



from steamship.plugin.parser import ParseRequest
from steamship.plugin.service import PluginRequest

from ..client.helpers import deploy_app, register_app_as_plugin, _steamship
from ..client.test_file_parse import parse_file

__copyright__ = "Steamship"
__license__ = "MIT"

TEST_REQ = ParseRequest(
    docs=["Hi there."],
    blockIds=["ABC"]
)
TEST_PLUGIN_REQ = PluginRequest(data=TEST_REQ)
TEST_REQ_DICT = TEST_PLUGIN_REQ.to_dict()


def test_e2e_embedder():
    client = _steamship()
    with deploy_app("plugin_parser.py") as (app, version, instance):
        with register_app_as_plugin(client, "parser", "parse", app, instance) as plugin:
            res = client.parse(docs=TEST_REQ.docs, plugin=plugin.handle)
            res.wait()
            assert (res.error is None)
            assert (res.data is not None)
            assert (len(res.data.blocks) == 1)
            assert (res.data.blocks[0].text == TEST_REQ.docs[0])

            # Let's try it on a file. This is the same test we run on the Swift test parser.
            # Since the python test parser is implemented to behave the same, we can reuse it!
            parse_file(client, plugin.handle)



from dataclasses import dataclass
from typing import List

from steamship.base import Client, Request


@dataclass
class Tag:
    tagId: str = None
    kind: str = None
    subKind: str = None
    name: str = None
    handle: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "Tag":
        return Tag(
            tagId=d.get('tagId', None),
            kind=d.get('kind', None),
            subKind=d.get('subKind', None),
            name=d.get('name', None),
            handle=d.get('handle', None),
        )


@dataclass
class CreateTagRequest(Request):
    tagId: str = None
    kind: str = None
    subKind: str = None
    name: str = None
    handle: str = None
    upsert: bool = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "CreateTagRequest":
        return CreateTagRequest(
            tagId=d.get('tagId', None),
            kind=d.get('kind', None),
            subKind=d.get('subKind', None),
            name=d.get('name', None),
            handle=d.get('handle', None),
            upsert=d.get('upsert', None),
        )


@dataclass
class ListTagsRequest(Request):
    objectType: str = None
    objectId: str = None


@dataclass
class DeleteTagRequest(Request):
    tagId: str = None
    kind: str = None
    subKind: str = None
    name: str = None
    handle: str = None
    upsert: bool = None


@dataclass
class TagObjectRequest(Request):
    tags: List[CreateTagRequest] = None
    objectType: str = None
    objectId: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "TagObjectRequest":
        tags = [CreateTagRequest.from_dict(dd) for dd in d.get('tags', [])]
        return TagObjectRequest(
            tags=tags,
            objectType=d.get('objectType', None),
            objectId=d.get('objectId', None),
        )


@dataclass
class UntagObjectRequest(Request):
    tags: List[CreateTagRequest] = None
    objectType: str = None
    objectId: str = None



from abc import ABC

from steamship.base import Client
from steamship.data.corpus import CorpusImportRequest, CorpusImportResponse
from steamship.plugin.service import PluginService, PluginRequest


class CorpusImporter(PluginService[CorpusImportRequest, CorpusImportResponse], ABC):
    @classmethod
    def subclass_request_from_dict(cls, d: any, client: Client = None) -> PluginRequest[CorpusImportRequest]:
        return CorpusImportRequest.from_dict(d, client=client)



import json
from abc import ABC
from dataclasses import dataclass
from typing import List

from steamship.base.client import Client
from steamship.plugin.service import PluginService, PluginRequest


@dataclass
class ClassifierHit:
    id: str = None
    value: str = None
    score: float = None
    externalId: str = None
    externalType: str = None
    metadata: any = None
    query: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "ClassifierHit":
        metadata = d.get("metadata", None)
        if metadata is not None:
            try:
                metadata = json.loads(metadata)
            except:
                pass

        return ClassifierHit(
            id=d.get("id", None),
            value=d.get("value", d.get("text", None)),
            score=d.get("score", None),
            externalId=d.get("externalId", None),
            externalType=d.get("externalType", None),
            metadata=metadata,
            query=d.get("query", None)
        )


@dataclass
class ClassifyRequest:
    docs: List[str]
    classifierId: str = None
    plugin: str = None
    labels: List[str] = None
    k: int = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "ClassifyRequest":
        return ClassifyRequest(
            type=d.get('type', None),
            plugin=d.get('plugin', None),
            id=d.get('id', None),
            handle=d.get('handle', None),
            name=d.get('name', None)
        )


@dataclass
class ClassifyResponse():
    classifierId: str = None
    plugin: str = None
    hits: List[List[ClassifierHit]] = None

    @staticmethod
    def from_dict(d: any = None, client: Client = None) -> "ClassifyResponse":
        hits = [[ClassifierHit.from_dict(h) for h in innerList] for innerList in (d.get("hits", []) or [])]
        return ClassifyResponse(
            classifierId=d.get('classifierId', None),
            plugin=d.get('plugin', None),
            hits=hits
        )


class Classifier(PluginService[ClassifyRequest, ClassifyResponse], ABC):
    @classmethod
    def subclass_request_from_dict(cls, d: any, client: Client = None) -> PluginRequest[ClassifyRequest]:
        return ClassifyRequest.from_dict(d, client=client)



from .app import App
from .app_instance import AppInstance
from .app_version import AppVersion
from .block import Block, BlockTypes
from .classifier import Classifier
from .corpus import Corpus
from .embedding_index import EmbeddingIndex
from .file import File
from .space import Space
from .span import Span
from .tag import Tag
from .token import Token



import io
import logging
import re
from typing import Union, Tuple, Any, List

from steamship.base import Client, Response, Request
from steamship.base.binary_utils import flexi_create
from steamship.base.request import IdentifierRequest
from steamship.data.block import Block
from steamship.data.converter import ClientsideConvertRequest, ConvertResponse
from steamship.data.embedding_index import EmbeddingIndex
from steamship.data.embedding_index import IndexItem
from steamship.data.parser import DependencyMatcher, PhraseMatcher, TokenMatcher
from steamship.data.parser import ParseRequest, ParseResponse
from steamship.data.plugin import PluginTargetType
from steamship.data.tag import CreateTagRequest, DeleteTagRequest, ListTagsRequest
from steamship.data.tag import TagObjectRequest

from dataclasses import dataclass


class FileUploadType:
    file = "file"
    url = "url"
    value = "value"
    fileImporter = "fileImporter"


_logger = logging.getLogger(__name__)


def parseDquery(query: str) -> List[Tuple[str, str, str]]:
    query = re.sub(' +', ' ', query.strip())
    parts = re.split(r'\s*(?=[@#])', query)
    ret = []

    for s in parts:
        s = s.strip()
        if not s:
            continue

        command = ''
        if s[0] in ['@', '#']:
            command = s[0]
            s = s[1:]

        if command == '':
            ret.append((command, None, s))
            continue

        if '"' not in s and ":" not in s:
            if command == '#':
                ret.append((command, 'contains', s))
            else:
                ret.append((command, s, None))
            continue

        modifier = None
        if ':' in s:
            ss = s.split(':')
            modifier = ss[0]
            s = ss[1]

        content = s
        if '"' in s:
            i = s.index('"')
            content = s[1 + i:-1]
            if modifier is None:
                s = s[:i]
                modifier = s
                if modifier == '':
                    modifier = None

        ret.append((command, modifier, content))
    return ret


@dataclass
class FileClearResponse:
    id: str


@dataclass
class FileTagRequest(Request):
    id: str
    plugin: str = None


@dataclass
class FileTagResponse:
    id: str
    tagResult: ParseResponse

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "FileTagResponse":
        if 'file' in d:
            d = d['file']
        return FileTagResponse(
            id=d.get('id', None),
            tagResult=ParseResponse.from_dict(d.get('tagResult', {}), client=client)
        )


@dataclass
class SpanQuery:
    text: str = None
    label: str = None
    spanType: str = None


@dataclass
class FileQueryRequest(Request):
    fileId: str
    type: str = None
    hasSpans: List[SpanQuery] = None
    text: str = None
    textMode: str = None
    isQuote: bool = None


@dataclass
class FileQueryResponse:
    id: str
    blocks: List[Block]

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "FileQueryResponse":
        return FileQueryResponse(
            id=d.get('id', None),
            blocks=[Block.from_dict(block, client=client) for block in d.get('blocks', None)]
        )


@dataclass
class FileRawRequest(Request):
    id: str


@dataclass
class ListFilesRequest(Request):
    corpusId: str = None


@dataclass
class ListFilesResponse:
    files: List["File"]

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "ListFilesResponse":
        return ListFilesResponse(
            files=[File.from_dict(f, client=client) for f in d.get('files', [])]
        )


@dataclass
class FileImportRequest:
    value: str = None
    data: str = None
    url: str = None
    type: str = None  # FileUploadType: fileImporter | value | url | data

    mimeType: str = None
    corpusId: str = None
    name: str = None

    plugin: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "FileImportRequest":
        return FileImportRequest(
            value=d.get('value', None),
            data=d.get('data', None),
            url=d.get('url', None),
            type=d.get('type', None),
            mimeType=d.get('mimeType', None),
            corpusId=d.get('corpusId', None),
            plugin=d.get('plugin', None),
            name=d.get('name', None)
        )

    def to_dict(self) -> dict:
        return dict(
            value=self.value,
            data=self.data,
            url=self.url,
            type=self.type,
            mimeType=self.mimeType,
            corpusId=self.corpusId,
            plugin=self.plugin,
            name=self.name
        )


@dataclass
class FileImportResponse:
    data: Any = None
    mimeType: str = None

    def __init__(
            self,
            data: Any = None,
            string: str = None,
            bytes: io.BytesIO = None,
            json: io.BytesIO = None,
            mimeType: str = None
    ):
        data, mimeType = flexi_create(
            body=data,
            string=string,
            json=json,
            bytes=bytes,
            mimeType=mimeType
        )
        self.data = data
        self.mimeType = mimeType

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "FileImportResponse":
        return FileImportResponse(
            data=d.get('data', None),
            mimeType=d.get('mimeType', None)
        )

    def to_dict(self) -> dict:
        return dict(
            data=self.data,
            mimeType=self.mimeType
        )


@dataclass
class File:
    """A file.
    """
    client: Client = None
    id: str = None
    name: str = None
    handle: str = None
    mimeType: str = None
    spaceId: str = None
    corpusId: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "File":
        if 'file' in d:
            d = d['file']
        return File(
            client=client,
            id=d.get('id', None),
            handle=d.get('handle', None),
            name=d.get('name', None),
            mimeType=d.get('mimeType', None),
            corpusId=d.get('corpusId', None),
            spaceId=d.get('spaceId', None)
        )

    def delete(
            self,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> "Response[File]":
        return self.client.post(
            'file/delete',
            IdentifierRequest(id=self.id),
            expect=File,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def clear(
            self,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> Response[FileClearResponse]:
        return self.client.post(
            'file/clear',
            IdentifierRequest(id=self.id),
            expect=FileClearResponse,
            ifdQuery=self,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    @staticmethod
    def upload(
            client: Client,
            filename: str = None,
            name: str = None,
            content: str = None,
            mimeType: str = None,
            corpusId: str = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ) -> "Response[File]":

        if filename is None and name is None and content is None:
            raise Exception("Either filename or name + content must be provided.")

        if filename is not None:
            with open(filename, 'rb') as f:
                content = f.read()
                name = filename

        req = FileImportRequest(
            type=FileUploadType.file,
            corpusId=corpusId,
            name=name,
            mimeType=mimeType
        )

        return client.post(
            'file/create',
            payload=req,
            file=(name, content, "multipart/form-data"),
            expect=File,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    @staticmethod
    def create(
            client: Client,
            filename: str = None,
            name: str = None,
            url: str = None,
            content: str = None,
            plugin: str = None,
            mimeType: str = None,
            corpusId: str = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ) -> "Response[File]":

        if filename is None and name is None and content is None and url is None and plugin is None:
            raise Exception("Either filename, name + content, url, or plugin must be provided.")

        if filename is not None:
            with open(filename, 'rb') as f:
                content = f.read()
                name = filename

        req = FileImportRequest(
            type=FileUploadType.fileImporter if plugin is not None else FileUploadType.file,
            corpusId=corpusId,
            name=name,
            url=url,
            mimeType=mimeType,
            plugin=plugin
        )

        return client.post(
            'file/create',
            payload=req,
            file=(name, content, "multipart/form-data"),
            expect=File,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    @staticmethod
    def list(
            client: Client,
            corpusId: str = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ):
        req = ListFilesRequest(
            corpusId=corpusId
        )
        res = client.post(
            'file/list',
            payload=req,
            expect=ListFilesResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )
        return res

    @staticmethod
    def scrape(
            client: Client,
            url: str,
            name: str = None,
            corpusId: str = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> "File":
        if name is None:
            name = url
        req = FileImportRequest(
            type=FileUploadType.url,
            name=name,
            url=url,
            corpusId=corpusId
        )

        return client.post(
            'file/create',
            payload=req,
            expect=File,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def convert(
            self,
            plugin: str = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None):
        req = ClientsideConvertRequest(
            id=self.id,
            type=PluginTargetType.file,
            plugin=plugin
        )

        return self.client.post(
            'plugin/convert',
            payload=req,
            expect=ConvertResponse,
            asynchronous=True,
            ifdQuery=self,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def parse(
            self,
            plugin: str = None,
            tokenMatchers: List[TokenMatcher] = None,
            phraseMatchers: List[PhraseMatcher] = None,
            dependencyMatchers: List[DependencyMatcher] = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ):
        req = ParseRequest(
            type=PluginTargetType.file,
            id=self.id,
            plugin=plugin,
            tokenMatchers=tokenMatchers,
            phraseMatchers=phraseMatchers,
            dependencyMatchers=dependencyMatchers
        )

        return self.client.post(
            'plugin/parse',
            payload=req,
            expect=ParseResponse,
            asynchronous=True,
            ifdQuery=self,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def tag(
            self,
            plugin: str = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ):
        req = FileTagRequest(
            id=self.id,
            plugin=plugin
        )

        return self.client.post(
            'file/tag',
            payload=req,
            expect=FileTagResponse,
            asynchronous=True,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def dquery(
            self,
            dQuery: str,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None):
        blockType = None
        hasSpans = []
        text = None
        isQuote = None
        textMode = None

        for tup in parseDquery(dQuery):
            (cmd, subcmd, content) = tup
            if cmd == '':
                blockType = content
            elif cmd == '#':
                text = content
                textMode = subcmd
            elif cmd == '@':
                hasSpans.append(SpanQuery(label=subcmd, text=content))

        return self.query(
            blockType=blockType,
            hasSpans=hasSpans,
            text=text,
            textMode=textMode,
            isQuote=isQuote,
            pd=True,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def query(
            self,
            blockType: str = None,
            hasSpans: List[SpanQuery] = None,
            text: str = None,
            textMode: str = None,
            isQuote: bool = None,
            pd: bool = False,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ) -> Response[FileQueryResponse]:

        req = FileQueryRequest(
            fileId=self.id,
            type=blockType,
            hasSpans=hasSpans,
            text=text,
            textMode=textMode,
            isQuote=isQuote
        )
        res = self.client.post(
            'file/query',
            payload=req,
            expect=FileQueryResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )
        return res

    def index(
            self,
            plugin: str = None,
            indexName: str = None,
            blockType: str = None,
            indexId: str = None,
            index: "EmbeddingIndex" = None,
            upsert: bool = True,
            reindex: bool = True,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> "EmbeddingIndex":
        # TODO: This should really be done all on the app, but for now we'll do it in the client
        # to facilitate demos.

        if indexId is None and index is not None:
            indexId = index.id

        if indexName is None:
            indexName = "{}-{}".format(self.id, plugin)

        if indexId is None and index is None:
            index = self.client.create_index(
                name=indexName,
                plugin=plugin,
                upsert=True,
                spaceId=spaceId,
                spaceHandle=spaceHandle,
                space=space
            ).data
        elif index is None:
            index = EmbeddingIndex(
                client=self.client,
                indexId=indexId
            )

        # We have an index available to us now. Perform the query.
        blocks = self.query(
            blockType=blockType,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        ).data.blocks

        items = []
        for block in blocks:
            item = IndexItem(
                value=block.text,
                externalId=block.id,
                externalType="block"
            )
            items.append(item)

        insert_task = index.insert_many(
            items,
            reindex=reindex,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

        insert_task.wait()
        return index

    def raw(
            self,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None):
        req = FileRawRequest(
            id=self.id,
        )

        return self.client.post(
            'file/raw',
            payload=req,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space,
            rawResponse=True
        )

    def add_tags(
            self,
            tags=List[Union[str, CreateTagRequest]],
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None):
        tagsNew = []
        for tag in tags:
            if type(tag) == str:
                tagsNew.append(CreateTagRequest(name=tag, upsert=True))
            elif type(tag) == CreateTagRequest:
                tagsNew.append(tag)
            else:
                raise (Exception("Unable to add tag of type: {}".format(type(tag))))

        req = TagObjectRequest(
            tags=tagsNew,
            objectType='file',
            objectId=self.id
        )

        return self.client.post(
            'tag/create',
            payload=req,
            expect=TagObjectRequest,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def remove_tags(
            self,
            tags=List[Union[str, DeleteTagRequest]],
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None):
        tagsNew = []
        for tag in tags:
            if type(tag) == str:
                tagsNew.append(DeleteTagRequest(name=tag))
            elif type(tag) == DeleteTagRequest:
                tagsNew.append(tag)
            else:
                raise (Exception("Unable to remove tag of type: {}".format(type(tag))))

        req = TagObjectRequest(
            tags=tagsNew,
            objectType='file',
            objectId=self.id
        )

        return self.client.post(
            'tag/delete',
            payload=req,
            expect=FileTagResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def list_tags(
            self,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None):
        req = ListTagsRequest(
            objectType='file',
            objectId=self.id
        )

        return self.client.post(
            'tag/list',
            payload=req,
            expect=TagObjectRequest,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )



from steamship import BlockTypes, File

from ..client.helpers import deploy_app, register_app_as_plugin, _steamship

__copyright__ = "Steamship"
__license__ = "MIT"


def test_e2e_converter():
    client = _steamship()
    with deploy_app("plugin_converter.py") as (app, version, instance):
        with register_app_as_plugin(client, "converter", "convert", app, instance) as plugin:
            file = File.upload(client=client, name="Test.txt", content="This is a test.").data
            assert (len(file.query(blockType=BlockTypes.H1).data.blocks) == 0)

            # Use the plugin we just registered
            file.convert(plugin=plugin.handle).wait()
            headers = file.query(blockType=BlockTypes.H1).data
            assert (len(file.query(blockType=BlockTypes.H1).data.blocks) == 1)
            assert (len(file.query(blockType=BlockTypes.Paragraph).data.blocks) == 2)

            file.delete()



from dataclasses import dataclass
from typing import List, Union

from steamship.base import Client
from steamship.data.span import Span
from steamship.data.token import Token


class BlockTypes:
    Document = "doc"
    Page = "page"  # E.g. in a PDF
    Region = "region"  # E.g., abstract catchall region in a document
    Header = "header"
    H1 = "h1"
    H2 = "h2"
    H3 = "h3"
    H4 = "h4"
    H5 = "h5"
    Line = "line"
    Title = "title"
    Subtitle = "subtitle"
    Footer = "footer"
    Paragraph = "paragraph"
    List = "list"
    Blockquote = "blockquote"
    Blockcode = "blockcode"
    Unk = "unk"
    Sentence = "sentence"
    Token = "token"
    Span = "span"


@dataclass
class Block:
    client: Client = None
    id: str = None
    type: str = None
    text: str = None
    children: List["Block"] = None
    tokens: List["Token"] = None
    spans: List["Span"] = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> Union["Block", None]:
        if d is None:
            return None
        if 'block' in d:
            d = d['block']
        return Block(
            client=client,
            id=d.get('id', None),
            type=d.get('type', None),
            text=d.get('text', None),
            children=list(map(lambda child: Block.from_dict(child, client=client), d.get('children', []))),
            tokens=list(map(lambda token: Token.from_dict(token, client=client), d.get('tokens', []))),
            # spans = list(map(lambda span: Span.from_dict(span, client=client), d.get('span', []))),
        )

    @staticmethod
    def from_spacy_root(
            id: str = None,
            spacyObj: any = None,
            includeTokens: bool = True,
            includeParseData: bool = True,
            includeEntities: bool = True
    ) -> "Block":
        children = None
        if spacyObj is not None:
            children = [
                Block.from_spacy(
                    type=BlockTypes.Sentence,
                    spacyObj=spacySent,
                    includeTokens=includeTokens,
                    includeParseData=includeParseData,
                    includeEntities=includeEntities
                )
                for spacySent in spacyObj.sents
            ]

        # spans = []
        # for label in d.spans:
        #   span_group = d.spans[label]
        #   for s in span_group:
        #     spans.append(Span.from_dict(s))

        # entities = []
        # if includeEntities is True:
        #   for ent in d.ents:
        #     e = Entity.from_spacy(ent)
        #     if e is not None:
        #       entities.append(e)

        return Block(
            id=id,
            type=BlockTypes.Document,
            children=children
        )

    @staticmethod
    def from_spacy(
            spacyObj: any = None,
            type: any = None,
            includeTokens: bool = True,
            includeParseData: bool = True,
            includeEntities: bool = True
    ) -> Union["Block", None]:

        if spacyObj is None:
            return None

        tokens = [
            Token.from_spacy(
                token,
                includeParseData=includeParseData
            ) for token in spacyObj] if includeTokens is True else []

        # entities = [Entity.from_spacy(e) for e in d.ents] if includeEntities is True else []

        return Block(
            type=type,
            text=spacyObj.text,
            tokens=tokens,
            spans=[]
        )

    def to_dict(self) -> dict:
        children = None
        if self.children is not None:
            children = [child.to_dict() for child in self.children]

        tokens = None
        if self.tokens is not None:
            tokens = [token.to_dict() for token in self.tokens]

        spans = None
        if self.spans is not None:
            spans = [span.to_dict() for span in self.spans]

        return dict(
            id=self.id,
            type=self.type,
            text=self.text,
            children=children,
            tokens=tokens,
            spans=spans
        )



from steamship import Block, BlockTypes, Token
from steamship.app import App, post, create_handler
from steamship.plugin.parser import Parser, ParseResponse, ParseRequest
from steamship.plugin.service import PluginResponse, PluginRequest


def _makeSentenceBlock(sentence: str, includeTokens: bool = True) -> Block:
    """Splits the sentence in tokens on space. Dep head of all is first token"""
    if includeTokens:
        tokens = [Token(text=word, parentIndex=0) for word in sentence.split(" ")]
        return Block(type=BlockTypes.Sentence, text=sentence, tokens=tokens)
    else:
        return Block(type=BlockTypes.Sentence, text=sentence)


def _makeDocBlock(text: str, blockId: str = None, includeTokens=True) -> Block:
    """Splits the document into sentences by assuming a period is a sentence divider."""
    # Add the period back
    sentences = list(map(lambda s: "{}.".format(s), text.split(".")))
    children = [_makeSentenceBlock(sentence, includeTokens=includeTokens) for sentence in sentences]
    return Block(id=blockId, text=text, type=BlockTypes.Document, children=children)


def _makeTestResponse(request: ParseRequest) -> ParseResponse:
    blocks = []
    for i, doc in enumerate(request.docs):
        # This is awkward and we shouldn're require plugin authors to return
        # the block ID below if possible... (Note: it's necessary for knowing
        # where in the DB to fold the results back into.)
        blockId = None
        if request.blockIds is not None and i < len(request.blockIds):
            i = request.blockIds[i]
        blocks.append(
            _makeDocBlock(
                doc,
                blockId=blockId,
                includeTokens=request.includeTokens is None or request.includeTokens is True
            )
        )
    response = ParseResponse(blocks=blocks)
    return response


class TestParserPlugin(Parser, App):
    # TODO: WARNING! We will need to implement some logic that prevents 
    # a distributed endless loop. E.g., a parser plugin returning the results
    # of using the Steamship client to call parse.. via itself!

    def run(self, request: PluginRequest[ParseRequest]) -> PluginResponse[ParseResponse]:
        if request.data is not None:
            return PluginResponse(
                data=_makeTestResponse(request.data)
            )

    @post('parse')
    def parse(self, **kwargs) -> dict:
        parseRequest = Parser.parse_request(request=kwargs)
        parseResponse = self.run(parseRequest)
        return Parser.response_to_dict(parseResponse)


handler = create_handler(TestParserPlugin)



from typing import Any


class SteamshipError(Exception):
    message: str = None
    suggestion: str = None
    code: str = None
    error: str = None

    def __init__(self, message: str = "Undefined remote error", suggestion: str = None, code: str = None,
                 error: Exception = None):
        self.message = message
        self.suggestion = suggestion
        self.code = code
        if error is not None:
            self.error = "{}".format(error)

        parts = []
        if code is not None:
            parts.append("[{}]".format(code))
        if message is not None:
            parts.append(message)
        if suggestion is not None:
            parts.append("Suggestion: {}".format(suggestion))

        super().__init__("\n".join(parts))

    @staticmethod
    def from_dict(d: any, client: Any = None) -> "SteamshipError":
        """Last resort if subclass doesn't override: pass through."""
        return SteamshipError(
            message=d.get('message', None),
            suggestion=d.get('suggestion', None),
            code=d.get('code', None)
        )



import pytest
from steamship import MimeTypes

from .helpers import _random_name, _steamship

__copyright__ = "Steamship"
__license__ = "MIT"


def test_corpus_create():
    steamship = _steamship()

    # Should require name
    with pytest.raises(Exception):
        corpus = steamship.create_corpus()

    corpus = steamship.create_corpus(name=_random_name()).data

    name_a = "{}.mkd".format(_random_name())
    a = corpus.upload(
        name=name_a,
        content="A",
        mimeType=MimeTypes.MKD
    ).data
    assert (a.id is not None)
    assert (a.name == name_a)
    assert (a.mimeType == MimeTypes.MKD)
    assert (a.corpusId == corpus.id)

    name_a = "{}.html".format(_random_name())
    a = corpus.scrape(
        name=name_a,
        url="https://edwardbenson.com/2020/10/gpt3-travel-agent"
    ).data
    assert (a.id is not None)
    assert (a.name == name_a)
    assert (a.corpusId == corpus.id)

    resp = corpus.list_files().data
    assert (len(resp.files) == 2)


def test_corpus_upsert():
    steamship = _steamship()
    name = _random_name()
    corpus1 = steamship.create_corpus(name=name).data
    assert (corpus1.id is not None)

    corpus2 = steamship.create_corpus(name=name)
    assert (corpus2.data is None)
    assert (corpus2.error is not None)

    corpus2 = steamship.create_corpus(name=name, handle=_random_name()).data
    assert (corpus1.id != corpus2.id)
    assert (corpus2.id is not None)

    corpus3 = steamship.create_corpus(name=name, upsert=True)
    assert (corpus3.error is None)
    assert (corpus3.data.id is not None)
    assert (corpus1.id == corpus3.data.id)


def test_corpus_delete():
    steamship = _steamship()
    name = _random_name()
    corpus1 = steamship.create_corpus(name=name).data
    resp = corpus1.delete()
    assert (resp.error is None)
    assert (resp.data is not None)
    assert (resp.data.id == corpus1.id)

    name_a = "{}.mkd".format(_random_name())
    a = corpus1.upload(
        name=name_a,
        content="A",
        mimeType=MimeTypes.MKD
    )
    assert (a.data is None)
    assert (a.error is not None)


def test_corpus_delete_cascade():
    steamship = _steamship()
    name = _random_name()
    corpus1 = steamship.create_corpus(name=name).data

    name_a = "{}.mkd".format(_random_name())
    a = corpus1.upload(
        name=name_a,
        content="A",
        mimeType=MimeTypes.MKD
    ).data

    res = a.query()

    corpus1.delete()
    res = a.query()
    assert (res.data is None)
    assert (res.error is not None)

# TODO: Add tests w/ operations in different spaces



import logging
from typing import List

from steamship.base import Client, Response
from steamship.client.tasks import Tasks
from steamship.data import Block, Classifier, Corpus, File
from steamship.data.embedding import EmbedAndSearchRequest, EmbedAndSearchResponse, EmbedRequest, EmbedResponse
from steamship.data.embedding_index import EmbeddingIndex
from steamship.data.parser import TokenMatcher, PhraseMatcher, DependencyMatcher, ParseRequest, ParseResponse
from steamship.data.space import Space
from steamship.data.tagging import TagRequest, TagResponse

__copyright__ = "Steamship"
__license__ = "MIT"

_logger = logging.getLogger(__name__)


class Steamship(Client):
    """Steamship Python Client."""

    def __init__(
            self,
            apiKey: str = None,
            apiBase: str = None,
            appBase: str = None,
            spaceId: str = None,
            spaceHandle: str = None,
            profile: str = None,
            configFile: str = None,
            configDict: dict = None,
            dQuery: bool = False):
        super().__init__(
            apiKey=apiKey,
            apiBase=apiBase,
            appBase=appBase,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            profile=profile,
            configFile=configFile,
            configDict=configDict,
            dQuery=dQuery)
        """
        The base.py class will properly detect and set the defaults. They should be None here.
    
        dQuery is a Beta option that will return chainable responses, like:
          file.upload()
              .convert()
              .parse()
              .embed()
              .query()
    
        It offers no new functionality -- in fact at the moment it's slightly less in that you 
        are given the syntactically convenient response object for chaining rather than the actual
        response object of the invocation.
        """
        self.tasks = Tasks(self)

    def create_corpus(
            self,
            name: str,
            handle: str = None,
            description: str = None,
            externalId: str = None,
            externalType: str = None,
            metadata: any = None,
            isPublic: bool = False,
            upsert: bool = False,
            spaceId: bool = False,
            spaceHandle: bool = False,
            space: Space = None
    ) -> Corpus:
        return Corpus.create(
            client=self,
            name=name,
            handle=handle,
            description=description,
            isPublic=isPublic,
            upsert=upsert,
            externalId=externalId,
            externalType=externalType,
            metadata=metadata,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def create_index(
            self,
            handle: str = None,
            name: str = None,
            plugin: str = None,
            upsert: bool = True,
            externalId: str = None,
            externalType: str = None,
            metadata: any = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: Space = None
    ) -> Response[EmbeddingIndex]:
        return EmbeddingIndex.create(
            client=self,
            handle=handle,
            name=name,
            plugin=plugin,
            upsert=upsert,
            externalId=externalId,
            externalType=externalType,
            metadata=metadata,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def create_classifier(
            self,
            name: str,
            plugin: str,
            upsert: bool = True,
            save: bool = None,
            labels: List[str] = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: Space = None
    ) -> Classifier:
        return Classifier.create(
            client=self,
            plugin=plugin,
            name=name,
            upsert=upsert,
            save=save,
            labels=labels,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def upload(
            self,
            filename: str = None,
            name: str = None,
            content: str = None,
            mimeType: str = None,
            plugin: str = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: Space = None
    ) -> File:
        return File.upload(
            self,
            filename=filename,
            name=name,
            content=content,
            mimeType=mimeType,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def scrape(
            self,
            url: str,
            name: str = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: Space = None,
    ) -> File:
        if name is None:
            name = url
        return File.scrape(
            self,
            url,
            name,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def embed(
            self,
            docs: List[str],
            plugin: str,
            spaceId: str = None,
            spaceHandle: str = None,
            space: Space = None
    ) -> Response[EmbedResponse]:
        req = EmbedRequest(
            docs=docs,
            plugin=plugin
        )
        return self.post(
            'embedding/create',
            req,
            expect=EmbedResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def embed_and_search(
            self,
            query: str,
            docs: List[str],
            plugin: str,
            k: int = 1,
            spaceId: str = None,
            spaceHandle: str = None,
            space: Space = None
    ) -> Response[EmbedAndSearchResponse]:
        req = EmbedAndSearchRequest(
            query=query,
            docs=docs,
            plugin=plugin,
            k=k
        )
        return self.post(
            'embedding/search',
            req,
            expect=EmbedAndSearchResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def parse(
            self,
            docs: List[str],
            plugin: str = None,
            tokenMatchers: List[TokenMatcher] = None,
            phraseMatchers: List[PhraseMatcher] = None,
            dependencyMatchers: List[DependencyMatcher] = None,
            includeTokens: bool = True,
            includeParseData: bool = True,
            includeEntities: bool = False,
            metadata: any = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: Space = None
    ) -> Response[ParseResponse]:
        req = ParseRequest(
            type="inline",
            docs=docs,
            plugin=plugin,
            tokenMatchers=tokenMatchers,
            phraseMatchers=phraseMatchers,
            dependencyMatchers=dependencyMatchers,
            includeTokens=includeTokens,
            includeParseData=includeParseData,
            includeEntities=includeEntities,
            metadata=metadata
        )
        return self.post(
            'plugin/parse',
            req,
            expect=ParseResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def tag(
            self,
            blocks: List[Block],
            plugin: str = None,
            metadata: any = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: Space = None
    ) -> Response[ParseResponse]:
        req = TagRequest(
            blocks=blocks,
            plugin=plugin,
            metadata=metadata
        )
        return self.post(
            'tagger/tag',
            req,
            expect=TagResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=Space
        )



from steamship import MimeTypes

from .helpers import _random_name, _steamship

__copyright__ = "Steamship"
__license__ = "MIT"


def test_file_tag():
    steamship = _steamship()
    name_a = "{}.mkd".format(_random_name())
    a = steamship.upload(
        name=name_a,
        content="A",
        mimeType=MimeTypes.MKD
    ).data
    assert (a.id is not None)
    assert (a.name == name_a)
    assert (a.mimeType == MimeTypes.MKD)

    a.add_tags(['test1', 'test2'])

    tags = a.list_tags().data.tags
    print(tags)
    assert (len(tags) == 2)

    must = ['test1', 'test2']
    for tag in tags:
        assert (tag.name in must)
        must.remove(tag.name)
    assert (len(must) == 0)

    a.remove_tags(['test1'])

    tags = a.list_tags().data.tags
    print(tags)
    assert (len(tags) == 1)

    must = ['test2']
    for tag in tags:
        assert (tag.name in must)
        must.remove(tag.name)
    assert (len(must) == 0)

    a.remove_tags(['test2'])
    tags = a.list_tags().data.tags
    print(tags)
    assert (len(tags) == 0)

    a.delete()



import logging
from dataclasses import dataclass
from typing import Dict

from steamship.base.configuration import Configuration


def event_to_config(event: dict) -> Configuration:
    if event is None:
        raise Exception("Null event provided")
    if "invocationContext" not in event:
        raise Exception("invocationContext not in event")

    return Configuration(from_dict=event["invocationContext"])


class Verb:
    GET = "GET"
    POST = "POST"

    @staticmethod
    def safely_from_str(s: str) -> str:
        ss = s.strip().upper()
        if ss == Verb.GET:
            return Verb.GET
        elif ss == Verb.POST:
            return Verb.POST
        return s


@dataclass
class Invocation:
    httpVerb: str = None
    appPath: str = None  # e.g. /hello/there
    arguments: Dict[str, any] = None

    @staticmethod
    def from_dict(d: dict) -> "Invocation":
        return Invocation(
            httpVerb=d.get('httpVerb', None),
            appPath=d.get('appPath', None),
            arguments=d.get('arguments', None)
        )


@dataclass
class Request:
    """An request of a method on an app instance.

    This is the payload sent from the public-facing App Proxy to the
    private-facing app microservice.
    """
    clientConfig: Configuration = None
    invocation: Invocation = None

    @staticmethod
    def from_dict(d: dict) -> "Request":
        invocation = Invocation.from_dict(d.get('invocation', dict()))
        clientConfig = Configuration.from_dict(d.get('clientConfig', dict()))
        logging.info("App layer got invocation {} {}".format(type(invocation), invocation))
        return Request(
            clientConfig=clientConfig,
            invocation=invocation
        )



from ..client.helpers import deploy_app, register_app_as_plugin, _steamship

__copyright__ = "Steamship"
__license__ = "MIT"

from ..client.test_embedding import basic_embeddings, basic_embedding_search
from ..client.test_embedding_index import create_index


def test_e2e_embedder():
    client = _steamship()
    with deploy_app("plugin_embedder.py") as (app, version, instance):
        with register_app_as_plugin(client, "embedder", "embed", app, instance) as plugin:
            e1 = client.embed(["This is a test"], plugin=plugin.handle)
            assert (e1.error is None)
            assert (len(e1.data.embeddings) == 1)
            assert (len(e1.data.embeddings[0]) > 1)

            e2 = client.embed(["This is a test"], plugin=plugin.handle)
            assert (e2.error is None)
            assert (len(e2.data.embeddings) == 1)
            assert (len(e2.data.embeddings[0]) == len(e1.data.embeddings[0]))

            e4 = client.embed(["This is a test"], plugin=plugin.handle)
            assert (e4.error is None)
            assert (len(e4.data.embeddings) == 1)

            # Now lets run all the other embedding tests
            basic_embeddings(client, plugin.handle)
            basic_embedding_search(client, plugin.handle)

            create_index(client, plugin.handle)



from abc import ABC

from steamship.base import Client
from steamship.data.embedding import EmbedRequest, EmbedResponse
from steamship.plugin.service import PluginService, PluginRequest


class Embedder(PluginService[EmbedRequest, EmbedResponse], ABC):
    @classmethod
    def subclass_request_from_dict(cls, d: any, client: Client = None) -> PluginRequest[EmbedRequest]:
        return EmbedRequest.from_dict(d, client=client)
